
/*******************************************************************************
 * Copyright (c) 2004, 2005 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/
package org.eclipse.birt.data.engine.olap.util;

import java.util.List;

import org.eclipse.birt.data.engine.api.IBaseExpression;

/**
 * This is an internal interface used to describe a measure that need to be calculated during 
 * Olap query execution.
 */

public interface ICubeAggrDefn
{
	/**
	 * Return the name of the cube aggregation definition. Usually it is a binding name.
	 * 
	 * @return
	 */
	public String getName();
	
	/**
	 * Return the measure that featured this aggregation.
	 * @return
	 */
	public String getMeasure();
	
	/**
	 * Return a list of levels that the aggregations is based.
	 * @return
	 */
	public List getAggrLevels();
	
	/**
	 * Return the name of the aggregation operation.
	 * @return
	 */
	public String getAggrName();
	
	/**
	 * Return a list of arguments that the aggregations is based.
	 * @return
	 */
	public List getArguments();
	
	/**
	 * Return FilterDefinition in aggregation definition
	 * 
	 * @return
	 */
	public IBaseExpression getFilter();
}
