/*******************************************************************************
 * Copyright (c) 2008 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.report.designer.ui.lib.explorer.dialog;

import java.util.Collection;
import java.util.Iterator;

import org.eclipse.birt.report.designer.internal.ui.dialogs.resource.ResourceFileContentProvider;
import org.eclipse.birt.report.designer.internal.ui.resourcelocator.ResourceEntry;
import org.eclipse.birt.report.designer.internal.ui.resourcelocator.ResourceEntryFilter;
import org.eclipse.birt.report.designer.internal.ui.resourcelocator.ResourceFilter;
import org.eclipse.birt.report.designer.ui.ReportPlugin;

/**
 * Tree viewer content provider adapter for resource folder.
 */
public class ResourceFolderProvider extends ResourceFileContentProvider
{

	/**
	 * Constructs a resource folder provider.
	 */
	public ResourceFolderProvider( )
	{
		super( false );
		setFilter( new ResourceEntry.Filter( ) {

			/** The filters of resource folder. */
			Collection<ResourceFilter> filters = ReportPlugin.getFilterMap( )
					.values( );

			{
				for ( Iterator<ResourceFilter> iterator = filters.iterator( ); iterator.hasNext( ); )
				{
					ResourceFilter filter = iterator.next( );

					if ( filter != null
							&& ResourceFilter.FILTER_EMPTY_FOLDERS.equals( filter.getType( ) ) )
					{
						// Doesn't filter empty folders.
						iterator.remove( );
					}
				}
			}

			/** The filter for the provider. */
			ResourceEntryFilter filter = new ResourceEntryFilter( (ResourceFilter[]) filters.toArray( new ResourceFilter[0] ) );

			/*
			 * (non-Javadoc)
			 * 
			 * @see org.eclipse.birt.report.designer.internal.ui.resourcelocator.ResourceEntry.Filter#accept(org.eclipse.birt.report.designer.internal.ui.resourcelocator.ResourceEntry)
			 */
			public boolean accept( ResourceEntry entity )
			{
				if ( !entity.isFile( ) )
				{
					return filter.accept( entity );
				}
				return false;
			}
		} );
	}
}
