/*******************************************************************************
 * Copyright (c) 2004 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.report.designer.internal.ui.dialogs;

import org.eclipse.birt.report.designer.nls.Messages;
import org.eclipse.birt.report.designer.ui.views.attributes.providers.ChoiceSetFactory;
import org.eclipse.birt.report.model.api.StyleHandle;
import org.eclipse.birt.report.model.api.elements.ReportDesignConstants;
import org.eclipse.birt.report.model.api.metadata.IChoice;
import org.eclipse.birt.report.model.api.metadata.IChoiceSet;
import org.eclipse.swt.layout.GridData;

/**
 * Preference page for Format style.
 */

public class FormatPreferencePage extends BaseStylePreferencePage
{

	private Object model;

	/**
	 * Default constructor.
	 * 
	 * @param model
	 *            The model of preference page.
	 */
	public FormatPreferencePage( Object model )
	{
		super( model );
		this.model = model;
		setTitle( Messages.getString( "FormatPreferencePage.displayname.Title" ) ); //$NON-NLS-1$
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.report.designer.internal.ui.dialogs.BaseStylePreferencePage#createFieldEditors()
	 */
	protected void createFieldEditors( )
	{
		super.createFieldEditors( );

		GridData gdata;

		EditableComboFieldEditor dateTime = new EditableComboFieldEditor( StyleHandle.DATE_TIME_FORMAT_PROP,
				Messages.getString( ( (StyleHandle) model ).getPropertyHandle( StyleHandle.DATE_TIME_FORMAT_PROP )
						.getDefn( )
						.getDisplayNameID( ) ),
				getChoiceArray( StyleHandle.DATE_TIME_FORMAT_PROP ),
				getFieldEditorParent( ) );
		gdata = new GridData( );
		gdata.widthHint = 200;
		dateTime.getComboBoxControl( getFieldEditorParent( ) )
				.setLayoutData( gdata );

		addField( dateTime );

		EditableComboFieldEditor number = new EditableComboFieldEditor( StyleHandle.NUMBER_FORMAT_PROP,
				Messages.getString( ( (StyleHandle) model ).getPropertyHandle( StyleHandle.NUMBER_FORMAT_PROP )
						.getDefn( )
						.getDisplayNameID( ) ),
				getChoiceArray( StyleHandle.NUMBER_FORMAT_PROP ),
				getFieldEditorParent( ) );
		gdata = new GridData( );
		gdata.widthHint = 200;
		number.getComboBoxControl( getFieldEditorParent( ) )
				.setLayoutData( gdata );

		addField( number );

		EditableComboFieldEditor string = new EditableComboFieldEditor( StyleHandle.STRING_FORMAT_PROP,
				Messages.getString( ( (StyleHandle) model ).getPropertyHandle( StyleHandle.STRING_FORMAT_PROP )
						.getDefn( )
						.getDisplayNameID( ) ),
				getChoiceArray( StyleHandle.STRING_FORMAT_PROP ),
				getFieldEditorParent( ) );
		gdata = new GridData( );
		gdata.widthHint = 200;
		string.getComboBoxControl( getFieldEditorParent( ) )
				.setLayoutData( gdata );

		addField( string );

	}

	private String[][] getChoiceArray( String propName )
	{
		IChoiceSet ci = ChoiceSetFactory.getElementChoiceSet( ReportDesignConstants.STYLE_ELEMENT,
				propName );

		if ( ci != null )
		{
			IChoice[] cs = ci.getChoices( );

			String[][] rt = new String[cs.length][2];

			for ( int i = 0; i < cs.length; i++ )
			{
				rt[i][0] = cs[i].getDisplayName( );
				rt[i][1] = cs[i].getName( );
			}

			return rt;
		}

		return new String[0][2];
	}

}