/*******************************************************************************
 * Copyright (c) 2006 Inetsoft Technology Corp.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Inetsoft Technology Corp  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.report.engine.emitter.wpml;

import java.io.OutputStream;
import java.text.Bidi;
import java.util.HashSet;
import java.util.List;
import java.util.StringTokenizer;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.eclipse.birt.report.engine.content.IAutoTextContent;
import org.eclipse.birt.report.engine.content.IReportContent;
import org.eclipse.birt.report.engine.content.IStyle;
import org.eclipse.birt.report.engine.css.engine.value.css.CSSConstants;
import org.eclipse.birt.report.engine.emitter.XMLWriter;
import org.eclipse.birt.report.model.api.core.IModuleModel;

public class WpmlWriter
{

	private int IMAGE_ID = 75;

	private int BOOKMARK_ID = 0;

	private int HEADING_ID = 0;

	public static int TOTALPAGE = 0;

	private XMLWriter writer = new XMLWriter( );

	public static int PAGE = 1;

	private static HashSet splitChar = new HashSet( );

	static
	{
		splitChar.add( new Character( ' ' ) );
		splitChar.add( new Character( '\r' ) );
		splitChar.add( new Character( '\n' ) );
	};

	public static enum TextFlag {
		start, middle, end, whole
	};
	protected static Logger logger = Logger.getLogger( WpmlWriter.class
			.getName( ) );

	// Holds the global layout orientation.
	private boolean rtl = false;

	// Holds Bidi text information
	private String[] bidiChunks = new String[0];
	private int[] bidiLevels = new int[0];

	public WpmlWriter( OutputStream out )
	{
		this( out, "UTF-8" );
	}

	public WpmlWriter( OutputStream out, String encoding )
	{
		writer.open( out, encoding );
	}

	public void start( IReportContent reportContent, boolean rtl )
	{
		this.rtl = rtl; // bidi_hcg

		writer.startWriter( );
		writer.literal( "\n" );
		writer.literal( "<?mso-application progid=\"Word.Document\"?>" );
		writer.literal( "\n" );
		writer.openTag( "w:wordDocument" );
		writer.attribute( "xmlns:w",
				"http://schemas.microsoft.com/office/word/2003/wordml" );
		writer.attribute( "xmlns:v", "urn:schemas-microsoft-com:vml" );
		writer.attribute( "xmlns:w10", "urn:schemas-microsoft-com:office:word" );
		writer.attribute( "xmlns:o", "urn:schemas-microsoft-com:office:office" );
		writer.attribute( "xmlns:dt",
				"uuid:C2F41010-65B3-11d1-A29F-00AA00C14882" );
		writer.attribute( "xmlns:wx",
				"http://schemas.microsoft.com/office/word/2003/auxHint" );
		writer.attribute( "xmlns:aml",
				"http://schemas.microsoft.com/aml/2001/core" );
		writer.attribute( "xml:space", "preserve" );
		writeDocumentProperties( reportContent );

		writer.openTag( "w:styles" );

		// style for outline
		writer.openTag( "w:style" );
		writer.attribute( "w:type", "paragraph" );
		writer.attribute( " w:styleId", 4 );
		writer.openTag( "w:name" );
		writer.attribute( "w:val", "toc 4" );
		writer.closeTag( "w:name" );
		writer.openTag( "wx:uiName" );
		writer.attribute( "wx:val", "catalog 4" );
		writer.closeTag( "wx:uiName" );
		writer.openTag( "w:autoRedefine" );
		writer.closeTag( "w:autoRedefine" );
		writer.openTag( "w:semiHidden" );
		writer.closeTag( "w:semiHidden" );
		writer.openTag( "w:rsid" );
		writer.attribute( "w:val", "009B3C8F" );
		writer.closeTag( "w:rsid" );
		writer.openTag( "w:pPr" );
		writer.openTag( "w:pStyle" );
		writer.attribute( "w:val", 4 );
		writer.closeTag( "w:pStyle" );

		writeBidi( rtl ); // bidi_hcg

		// writer.openTag( "w:tabs" );
		// writer.openTag( "w:tab" );
		// writer.attribute( "w:val", "right" );
		// writer.attribute( "w:leader", "dot" );
		// writer.attribute( "w:pos", pagewidth );
		// writer.closeTag( "w:tab" );
		// writer.closeTag( "w:tabs" );

		writer.closeTag( "w:pPr" );
		writer.openTag( "w:rPr" );
		writer.openTag( "wx:font" );
		writer.attribute( "wx:val", "Times New Roman" );
		writer.closeTag( "wx:font" );
		writer.closeTag( "w:rPr" );
		writer.closeTag( "w:style" );
		writer.closeTag( "w:styles" );
		// For show background
		writer.openTag( "w:displayBackgroundShape" );
		writer.closeTag( "w:displayBackgroundShape" );

		writer.openTag( "w:docPr" );
		writer.openTag( "w:view" );
		writer.attribute( "w:val", "print" );
		writer.closeTag( "w:view" );
		writer.openTag( "w:zoom" );
		writer.attribute( "w:percent", "100" );
		writer.closeTag( "w:zoom" );
		writer.closeTag( "w:docPr" );
		writer.openTag( "w:body" );

	}

	private void writeDocumentProperties( IReportContent reportContent )
	{
		writer.openTag( "o:DocumentProperties" );
		writer.openTag( "o:Author" );
		writer.text( reportContent.getDesign( ).getReportDesign( )
				.getStringProperty( IModuleModel.AUTHOR_PROP ) );
		writer.closeTag( "o:Author" );

		writer.openTag( "o:Title" );
		writer.text( reportContent.getDesign( ).getReportDesign( )
				.getStringProperty( IModuleModel.TITLE_PROP ) );
		writer.closeTag( "o:Title" );

		writer.openTag( "o:Description" );
		writer.text( reportContent.getDesign( ).getReportDesign( )
				.getStringProperty( IModuleModel.DESCRIPTION_PROP ) );
		writer.closeTag( "o:Description" );

		writer.closeTag( "o:DocumentProperties" );
	}

	public void startPageBreak( )
	{
		writer.openTag( "w:p" );
		writer.openTag( "w:pPr" );
	}

	public void endPageBreak( )
	{
		writer.closeTag( "w:pPr" );
		writer.closeTag( "w:p" );
	}

	public void startSection( )
	{
		writer.openTag( "w:sectPr" );
	}

	public void endSection( )
	{
		writer.closeTag( "w:sectPr" );
	}

	public void writeTocs( List tocs )
	{
		writer.openTag( "wx:sub-section" );
		int count = tocs.size( );

		for ( int i = 0; i < count; i++ )
		{
			String toc = (String) tocs.get( i );
			writeTOC( toc, toc );
		}

		writer.closeTag( "wx:sub-section" );

		// now comes the content
		// writer.openTag( "wx:sub-section" );

	}

	// public void endBodyContent() {
	// writer.closeTag("wx:sub-section");
	// }
	//	
	private void writeTOC( String head, String target )
	{
		head = ridquote( head );
		target = WordUtil.validBookmarkName( ridquote( target ) );

		writer.openTag( "w:p" );

		writer.openTag( "w:pPr" );
		writer.openTag( "w:pStyle" );
		writer.attribute( "w:val", 4 );
		writer.closeTag( "w:pStyle" );
		writer.openTag( "w:rPr" );
		writer.openTag( "w:noProof" );
		writer.closeTag( "w:noProof" );
		writer.closeTag( "w:rPr" );
		writer.closeTag( "w:pPr" );

		writer.openTag( "w:r" );
		writer.openTag( "w:rPr" );
		writer.openTag( "w:webHidden" );
		writer.closeTag( "w:webHidden" );
		writer.closeTag( "w:rPr" );
		writer.openTag( "w:fldChar" );
		writer.attribute( "w:fldCharType", "begin" );
		writer.closeTag( "w:fldChar" );
		writer.closeTag( "w:r" );

		writer.openTag( "w:r" );
		writer.openTag( "w:instrText" );
		writer.text( "TOC \\o \"1-3\" \\h \\z \\t" );
		writer.closeTag( "w:instrText" );
		writer.closeTag( "w:r" );

		writer.openTag( "w:r" );
		writer.openTag( "w:instrText" );
		writer.text( "content" );
		writer.closeTag( "w:instrText" );
		writer.closeTag( "w:r" );

		writer.openTag( "w:r" );
		writer.openTag( "w:instrText" );
		writer.text( "��4\"" );
		writer.closeTag( "w:instrText" );
		writer.closeTag( "w:r" );

		writer.openTag( "w:r" );
		writer.openTag( "w:rPr" );
		writer.openTag( "w:webHidden" );
		writer.closeTag( "w:webHidden" );
		writer.closeTag( "w:rPr" );
		writer.openTag( "w:fldChar" );
		writer.attribute( "w:fldCharType", "separate" );
		writer.closeTag( "w:fldChar" );
		writer.closeTag( "w:r" );

		writer.openTag( "w:hlink" );
		writer.attribute( "w:bookmark", target );

		writer.openTag( "w:r" );
		writer.openTag( "w:rPr" );
		writer.openTag( "w:rStyle" );
		writer.attribute( "w:val", "a4" );
		writer.closeTag( "w:rStyle" );
		writer.openTag( "w:rFonts" );
		writer.attribute( "w:hint", "fareast" );
		writer.closeTag( "w:rFonts" );
		writer.openTag( "w:noProof" );
		writer.closeTag( "w:noProof" );
		writer.closeTag( "w:rPr" );
		writer.openTag( "w:t" );
		writer.text( head );
		writer.closeTag( "w:t" );
		writer.closeTag( "w:r" );

		writer.openTag( "w:r" );
		writer.openTag( "w:rPr" );
		writer.openTag( "w:noProof" );
		writer.closeTag( "w:noProof" );
		writer.openTag( "w:webHidden" );
		writer.closeTag( "w:webHidden" );
		writer.closeTag( "w:rPr" );
		writer.openTag( "w:tab" );
		writer.attribute( "wx:wTab", 5 );
		writer.attribute( "wx:tlc", "dot" );
		writer.attribute( "wx:cTlc", 0 );
		writer.closeTag( "w:tab" );
		writer.closeTag( "w:r" );

		writer.openTag( "w:r" );
		writer.openTag( "w:rPr" );
		writer.openTag( "w:noProof" );
		writer.closeTag( "w:noProof" );
		writer.openTag( "w:webHidden" );
		writer.closeTag( "w:webHidden" );
		writer.closeTag( "w:rPr" );
		writer.openTag( "w:fldChar" );
		writer.attribute( "w:fldCharType", "begin" );
		writer.openTag( "w:fldData" );
		writer.text( "CNDJ6nn5us4RjIIAqgBLqQsCAAAACAAAAA4AAABfAFQAbwBjADEANAA1"
				+ "ADMAOAA3ADcANAA3AAAA" );
		writer.closeTag( "w:fldData" );
		writer.closeTag( "w:fldChar" );
		writer.closeTag( "w:r" );

		writer.openTag( "w:r" );
		writer.openTag( "w:rPr" );
		writer.openTag( "w:noProof" );
		writer.closeTag( "w:noProof" );
		writer.openTag( "w:webHidden" );
		writer.closeTag( "w:webHidden" );
		writer.closeTag( "w:rPr" );
		writer.openTag( "w:instrText" );
		writer.text( "PAGEREF  " + target + "\\h" );
		writer.closeTag( "w:instrText" );
		writer.closeTag( "w:r" );

		writer.openTag( "w:r" );
		writer.openTag( "w:rPr" );
		writer.openTag( "w:noProof" );
		writer.closeTag( "w:noProof" );
		writer.closeTag( "w:rPr" );
		writer.closeTag( "w:r" );

		writer.openTag( "w:r" );
		writer.openTag( "w:rPr" );
		writer.openTag( "w:noProof" );
		writer.closeTag( "w:noProof" );
		writer.openTag( "w:webHidden" );
		writer.closeTag( "w:webHidden" );
		writer.closeTag( "w:rPr" );
		writer.openTag( "w:fldChar" );
		writer.attribute( "w:fldCharType", "separate" );
		writer.closeTag( "w:fldChar" );
		writer.closeTag( "w:r" );

		writer.openTag( "w:r" );
		writer.openTag( "w:rPr" );
		writer.openTag( "w:noProof" );
		writer.closeTag( "w:noProof" );
		writer.openTag( "w:webHidden" );
		writer.closeTag( "w:webHidden" );
		writer.closeTag( "w:rPr" );
		writer.openTag( "w:t" );
		writer.text( "1" );
		writer.closeTag( "w:t" );
		writer.closeTag( "w:r" );

		writer.openTag( "w:r" );
		writer.openTag( "w:rPr" );
		writer.openTag( "w:noProof" );
		writer.closeTag( "w:noProof" );
		writer.openTag( "w:webHidden" );
		writer.closeTag( "w:webHidden" );
		writer.closeTag( "w:rPr" );
		writer.openTag( "w:fldChar" );
		writer.attribute( "w:fldCharType", "end" );
		writer.closeTag( "w:fldChar" );
		writer.closeTag( "w:r" );

		writer.closeTag( "w:hlink" );

		// to match the declared toc
		writer.openTag( "w:r" );
		writer.openTag( "w:rPr" );
		writer.openTag( "w:noProof" );
		writer.closeTag( "w:noProof" );
		writer.openTag( "w:webHidden" );
		writer.closeTag( "w:webHidden" );
		writer.closeTag( "w:rPr" );
		writer.openTag( "w:fldChar" );
		writer.attribute( "w:fldCharType", "end" );
		writer.closeTag( "w:fldChar" );
		writer.closeTag( "w:r" );

		writer.closeTag( "w:p" );
	}

	// write the table properties to the output stream
	public void startTable( IStyle style, int tablewidth )
	{
		writer.openTag( "w:tbl" );
		writer.openTag( "w:tblPr" );
		writer.openTag( "w:tblInd" );
		writer.attribute( "w:w", 0 );
		writer.attribute( "w:type", "dxa" );
		writer.closeTag( "w:tblInd" );
		writeAttrTag( "w:tblStyle", "TableGrid" );
		writeAttrTag( "w:tblOverlap", "Never" );

		writeBidiTable( );

		writer.openTag( "w:tblW" );
		writer.attribute( "w:w", tablewidth );
		writer.attribute( "w:type", "dxa" );
		writer.closeTag( "w:tblW" );
		writeAttrTag( "w:tblLook", "01E0" );
		writer.openTag( "w:tblLayout" );
		writer.attribute( "w:type", "Fixed" );
		writer.closeTag( "w:tblLayout" );

		writer.openTag( "w:tblBorders" );
		writeBorders( style );
		writer.closeTag( "w:tblBorders" );

		String color = parseColor( style.getBackgroundColor( ) );
		writeBackgroundColor( color );
		writeAlign( style.getTextAlign( ), style.getDirection( ) );
		writer.closeTag( "w:tblPr" );
	}

	private void writeSpacing( String height )
	{
		writer.openTag( "w:spacing" );
		writer.attribute( "w:before", ( Math
				.round( Float.parseFloat( height ) / 1000 ) * 20 )
				/ 2 + "" );
		writer.attribute( "w:after", ( Math
				.round( Float.parseFloat( height ) / 1000 ) * 20 )
				/ 2 + "" );

		writer.closeTag( "w:spacing" );

	}

	private void writeSpacing( String top, String bottom )
	{
		writer.openTag( "w:spacing" );
		writer.attribute( "w:before", ( Math
				.round( Float.parseFloat( top ) / 1000 ) * 20 )
				/ 2 + "" );
		writer.attribute( "w:after", ( Math
				.round( Float.parseFloat( bottom ) / 1000 ) * 20 )
				/ 2 + "" );

		writer.closeTag( "w:spacing" );

	}

	private void writeAlign( String align, String direction )
	{
		if ( null == align )
		{
			return;
		}
		if ( "justify".equalsIgnoreCase( align ) )
		{
			align = "both";
		}
		// Need to swap 'left' and 'right' when orientation is RTL.
		if ( CSSConstants.CSS_RTL_VALUE.equalsIgnoreCase( direction ) )
		{
			if ( IStyle.CSS_RIGHT_VALUE.equals( align ) )
				writeAttrTag( "w:jc", IStyle.CSS_LEFT_VALUE );
			else if ( IStyle.CSS_LEFT_VALUE.equals( align ) )
				writeAttrTag( "w:jc", IStyle.CSS_RIGHT_VALUE );
			else
				writeAttrTag( "w:jc", align );
		}
		else
			writeAttrTag( "w:jc", align );
	}

	/**
	 * @param direction
	 * 
	 * @author bidi_hcg
	 */
	private void writeBidi( boolean rtl )
	{
		if ( rtl )
		{
			writer.openTag( "w:bidi" );
			writer.closeTag( "w:bidi" );
		}
		else
		{
			writeAttrTag( "w:bidi", "off" );
		}
	}

	/**
	 * @param direction
	 * 
	 * @author bidi_hcg
	 */
	private void writeBidiTable( )
	{
		if ( this.rtl )
		{
			writer.openTag( "w:bidiVisual" );
			writer.closeTag( "w:bidiVisual" );
		}
	}

	private void writeBorders( IStyle style )
	{
		String color = style.getBorderBottomColor( );
		String bstyle = style.getBorderBottomStyle( );
		String width = style.getBorderBottomWidth( );
		writeSingleBorder( "bottom", color, bstyle, width );

		color = style.getBorderTopColor( );
		bstyle = style.getBorderTopStyle( );
		width = style.getBorderTopWidth( );

		writeSingleBorder( "top", color, bstyle, width );

		color = style.getBorderLeftColor( );
		bstyle = style.getBorderLeftStyle( );
		width = style.getBorderLeftWidth( );

		writeSingleBorder( "left", color, bstyle, width );

		color = style.getBorderRightColor( );
		bstyle = style.getBorderRightStyle( );
		width = style.getBorderRightWidth( );

		writeSingleBorder( "right", color, bstyle, width );
	}

	private void writeSingleBorder( String type, String color, String style,
			String width )
	{
		// If the border style isn't set,
		// then guess this border is not set
		if ( style == null || "none".equalsIgnoreCase( style ) )
		{
			return;
		}

		writer.openTag( "w:" + type );
		writeBorderProperty( color, style, width );
		writer.closeTag( "w:" + type );
	}

	private void writeBorderProperty( String color, String style, String width )
	{
		style = getBorderStyle( style );

		int sz = getBorderSize( width );
		writer.attribute( "w:val", style );
		writer.attribute( "w:sz", sz );
		writer.attribute( "wx:bdrwidth", getBorderWidth( sz, style ) );
		writer.attribute( "w:space", 0 );
		writer.attribute( "w:color", parseColor( color ) );
	}

	private String getBorderStyle( String style )
	{
		if ( "solid".equalsIgnoreCase( style ) )
		{
			style = "single";
		}

		return style;
	}

	private int getBorderWidth( int sz, String style )
	{
		if ( sz == 0 )
		{
			return 0;
		}
		else
		{
			int ratio = 1;

			if ( "double".equalsIgnoreCase( style ) )
			{
				ratio = 3;
			}

			return ( sz / 2 ) * 5 * ratio;
		}
	}

	private int getBorderSize( String width )
	{
		int sz = 0;

		try
		{
			int w = Math.round( Float.parseFloat( width ) );

			sz = ( 8 * w ) / 750;
			return sz;
		}
		catch ( Exception e )
		{
			logger.log( Level.WARNING, e.getMessage( ), e );
		}

		return sz;
	}

	private int getImageID( )
	{
		return IMAGE_ID++;
	}

	/**
	 * 
	 * @param data
	 *            image data
	 * @param height
	 *            image height, unit = pt
	 * @param width
	 *            image width, unit = pt
	 */

	public void drawImage( String data, double height, double width,
			HyperlinkInfo hyper, IStyle style, boolean inline,
			boolean first_inline, String altText )
	{
		if ( !inline || first_inline )
		{
			writer.openTag( "w:p" );
		}

		int imageId = getImageID( );
		openHyperlink( hyper );
		writer.openTag( "w:r" );
		writer.openTag( "w:pict" );
		writer.openTag( "v:shapetype" );
		writer.attribute( "id", "_x0000_t" + imageId );
		writer.attribute( "coordsize", "21600,21600" );
		writer.attribute( "o:spt", "75" );
		writer.attribute( "o:preferrelative", "t" );
		writer.attribute( "path", "m@4@5l@4@11@9@11@9@5xe" );
		writer.attribute( "filled", "f" );
		writer.attribute( "stroked", "f" );
		writer.openTag( "v:stroke" );
		writer.attribute( "joinstyle", "miter" );
		writer.closeTag( "v:stroke" );
		writer.openTag( "v:formulas" );
		writer.openTag( "v:f" );
		writer.attribute( "eqn", "if lineDrawn pixelLineWidth 0" );
		writer.closeTag( "v:f" );
		writer.openTag( "v:f" );
		writer.attribute( "eqn", "sum @0 1 0" );
		writer.closeTag( "v:f" );
		writer.openTag( "v:f" );
		writer.attribute( "eqn", "sum 0 0 @1" );
		writer.closeTag( "v:f" );
		writer.openTag( "v:f" );
		writer.attribute( "eqn", "prod @2 1 2" );
		writer.closeTag( "v:f" );
		writer.openTag( "v:f" );
		writer.attribute( "eqn", "prod @3 21600 pixelWidth" );
		writer.closeTag( "v:f" );
		writer.openTag( "v:f" );
		writer.attribute( "eqn", "prod @3 21600 pixelHeight" );
		writer.closeTag( "v:f" );
		writer.openTag( "v:f" );
		writer.attribute( "eqn", "sum @0 0 1" );
		writer.closeTag( "v:f" );
		writer.openTag( "v:f" );
		writer.attribute( "eqn", "prod @6 1 2" );
		writer.closeTag( "v:f" );
		writer.openTag( "v:f" );
		writer.attribute( "eqn", "prod @7 21600 pixelWidth" );
		writer.closeTag( "v:f" );
		writer.openTag( "v:f" );
		writer.attribute( "eqn", "sum @8 21600 0 " );
		writer.closeTag( "v:f" );
		writer.openTag( "v:f" );
		writer.attribute( "eqn", "prod @7 21600 pixelHeight" );
		writer.closeTag( "v:f" );
		writer.openTag( "v:f" );
		writer.attribute( "eqn", "sum @10 21600 0" );
		writer.closeTag( "v:f" );
		writer.closeTag( "v:formulas" );
		writer.openTag( "v:path" );
		writer.attribute( "o:extrusionok", "f" );
		writer.attribute( "gradientshapeok", "t" );
		writer.attribute( "o:connecttype", "rect" );
		writer.closeTag( "v:path" );
		writer.openTag( "o:lock" );
		writer.attribute( "v:ext", "edit" );
		writer.attribute( "aspectratio", "t" );
		writer.closeTag( "o:lock" );

		writer.closeTag( "v:shapetype" );
		if ( data != null )
		{
			writer.openTag( "w:binData" );
			writer.attribute( "w:name", "wordml://" + imageId + ".png" );
			writer.text( data );
			writer.closeTag( "w:binData" );
		}
		writer.openTag( "v:shape" );
		writer.attribute( "id", "_x0000_i10" + imageId );
		writer.attribute( "type", "#_x0000_t" + imageId );
		writer.attribute( "alt", altText );
		writer.attribute( "style", "width:" + width + "pt;height:" + height
				+ "pt" );
		writeImageBordersColor( style );
		writer.openTag( "v:imagedata" );
		writer.attribute( "src", "wordml://" + imageId + ".png" );
		writer.attribute( "otitle", "" );
		writer.closeTag( "v:imagedata" );
		writeImageBorderStyle( style );
		writer.closeTag( "v:shape" );
		writer.closeTag( "w:pict" );
		writer.closeTag( "w:r" );
		closeHyperlink( hyper );
		if ( !inline )
		{
			writer.closeTag( "w:p" );
		}
	}

	private void writeImageBordersColor( IStyle style )
	{
		String color = style.getBorderBottomColor( );
		writeImageBorderColor( "bottom", color );

		color = style.getBorderTopColor( );
		writeImageBorderColor( "top", color );

		color = style.getBorderLeftColor( );
		writeImageBorderColor( "left", color );

		color = style.getBorderRightColor( );
		writeImageBorderColor( "right", color );
	}

	private void writeImageBorderColor( String pos, String color )
	{
		String direct = "o:border" + pos + "color";
		writer.attribute( direct, color );
	}

	private void writeImageBorderStyle( IStyle style )
	{
		String bstyle = style.getBorderBottomStyle( );
		String width = style.getBorderBottomWidth( );
		writeImgBorderStyle( "bottom", bstyle, width );

		bstyle = style.getBorderTopStyle( );
		width = style.getBorderTopWidth( );
		writeImgBorderStyle( "top", bstyle, width );

		bstyle = style.getBorderLeftStyle( );
		width = style.getBorderLeftWidth( );
		writeImgBorderStyle( "left", bstyle, width );

		bstyle = style.getBorderRightStyle( );
		width = style.getBorderRightWidth( );
		writeImgBorderStyle( "right", bstyle, width );
	}

	private void writeImgBorderStyle( String pos, String style, String width )
	{
		String direct = "w10:border" + pos;
		writer.openTag( direct );
		writer.attribute( "type", getBorderStyle( style ) );

		int sz = getBorderSize( width );
		writer.attribute( "width", getBorderWidth( sz, width ) );
		writer.closeTag( direct );

	}

	public void writeCol( int[] cols )
	{
		writer.openTag( "w:tblGrid" );

		for ( int i = 0; i < cols.length; i++ )
		{
			writeAttrTag( "w:gridCol", cols[i] + "" );
		}
		writer.closeTag( "w:tblGrid" );
	}

	public void endTable( )
	{
		writer.closeTag( "w:tbl" );
	}

	public void insertHiddenP( )
	{
		writer.openTag( "w:p" );
		writer.openTag( "w:rPr" );
		writeAttrTag( "w:vanish", "on" );
		writer.closeTag( "w:rPr" );
		writer.closeTag( "w:p" );
	}

	// Word seems not supporting a text margin and padding
	// those properties will be ignore.
	public void writeText( String txt, IStyle style, int w )
	{
		writeContent( WordEmitterImpl.NORMAL, txt, style,
				style.getFontFamily( ), null, false, false, TextFlag.whole );
	}

	public void writeContent( int type, String txt, IStyle style,
			String fontFamily, HyperlinkInfo info, boolean inline,
			boolean first, TextFlag flag )
	{

		if ( inline )
		{
			if ( first && flag == TextFlag.start )
			{
				startParagraphStyle( style );
			}

			writeTextInR( type, txt, style, fontFamily, info, true );
		}
		else
		{
			writeTextInP( type, txt, style, fontFamily, info, flag );
		}

	}

	private void writeTextInR( int type, String txt, IStyle style,
			String fontFamily, HyperlinkInfo info, boolean prop )
	{
		if ( "".equals( txt ) )
		{
			return;
		}
		openHyperlink( info );
		boolean reserved = isReserved( type );

		String direction = style.getDirection( );
		boolean textIsRtl = CSSConstants.CSS_RTL_VALUE.equals( direction );
		int nChunks = reserved ? 1 : 0;
		if ( "\n".equals( txt ) || "\r".equalsIgnoreCase( txt )
				|| "\r\n".equals( txt ) )
		{
			writer.closeTag( "w:p" );
			startParagraphStyle( style );
			return;
		}
		writer.openTag( "w:pPr" );
		writeBidi( textIsRtl );
		writer.closeTag( "w:pPr" );

		if ( reserved )
		{
			writer.openTag( "w:r" );
			writer.openTag( "w:fldChar" );
			writer.attribute( "w:fldCharType", "begin" );
			writer.closeTag( "w:fldChar" );
			writer.closeTag( "w:r" );
		}

		else
		{
			buildBidiChunks( txt, textIsRtl );
			nChunks = bidiChunks.length;
			assert nChunks > 0;
		}

		for ( int i = 0; i < nChunks; i++ )
		{
			writer.openTag( "w:r" );
			writer.openTag( "w:rPr" );
			writeRunProperties( style, fontFamily, info != null );

			if ( prop )
			{
				writeAlign( style.getTextAlign( ), direction );
				String val = parseColor( style.getBackgroundColor( ) );
				writeBackgroundColor( val );
				writeRunBorders( style );
			}

			if ( !reserved && bidiLevels[i] == Bidi.DIRECTION_RIGHT_TO_LEFT )
			{
				writer.openTag( "w:rtl" );
				writer.closeTag( "w:rtl" );
			}
			writer.closeTag( "w:rPr" );

			if ( reserved )
			{
				writeAutoText( type );
			}
			else
			{
				writeString( bidiChunks[i], style );
			}

			writer.closeTag( "w:r" );
		}
		if ( reserved )
		{
			writer.openTag( "w:r" );
			writer.openTag( "w:fldChar" );
			writer.attribute( "w:fldCharType", "end" );
			writer.closeTag( "w:fldChar" );
			writer.closeTag( "w:r" );
		}

		closeHyperlink( info );
	}

	private void startParagraphStyle( IStyle style )
	{
		writer.openTag( "w:p" );
		writer.openTag( "w:pPr" );
		writeSpacing( ( style.getProperty( IStyle.STYLE_MARGIN_TOP ) )
				.getCssText( ), ( style
				.getProperty( IStyle.STYLE_MARGIN_BOTTOM ) ).getCssText( ) );
		writeAlign( style.getTextAlign( ), style.getDirection( ) );
		String val = parseColor( style.getBackgroundColor( ) );
		writeBackgroundColor( val );
		writer.closeTag( "w:pPr" );
	}

	private boolean isReserved( int type )
	{
		return type == IAutoTextContent.PAGE_NUMBER
				|| type == IAutoTextContent.TOTAL_PAGE;
	}

	private void writeRunBorders( IStyle style )
	{
		String val = style.getBorderTopStyle( );

		if ( !( val == null || "none".equalsIgnoreCase( val ) ) )
		{
			writer.openTag( "w:bdr" );
			writeBorderProperty( style.getBorderTopColor( ), val, style
					.getBorderTopWidth( ) );
			writer.closeTag( "w:bdr" );
			return;
		}

		val = style.getBorderBottomStyle( );
		if ( !( val == null || "none".equalsIgnoreCase( val ) ) )
		{
			writer.openTag( "w:bdr" );
			writeBorderProperty( style.getBorderBottomColor( ), val, style
					.getBorderBottomWidth( ) );
			writer.closeTag( "w:bdr" );
			return;
		}

		val = style.getBorderLeftStyle( );
		if ( !( val == null || "none".equalsIgnoreCase( val ) ) )
		{
			writer.openTag( "w:bdr" );
			writeBorderProperty( style.getBorderLeftColor( ), val, style
					.getBorderLeftWidth( ) );
			writer.closeTag( "w:bdr" );
			return;
		}

		val = style.getBorderRightStyle( );
		if ( !( val == null || "none".equalsIgnoreCase( val ) ) )
		{
			writer.openTag( "w:bdr" );
			writeBorderProperty( style.getBorderRightColor( ), val, style
					.getBorderRightWidth( ) );
			writer.closeTag( "w:bdr" );
			return;
		}
	}

	private void writeTextInP( int type, String txt, IStyle style,
			String fontFamily, HyperlinkInfo info, TextFlag flag )
	{
		if ( flag == TextFlag.start )
		{
			outputText( type, txt, style, fontFamily, info );
		}
		else if ( flag == TextFlag.end )
		{
			writer.closeTag( "w:p" );
		}
		else if ( flag == TextFlag.middle )
		{
			writeTextInR( type, txt, style, fontFamily, info, false );
		}
		else
		{
			outputText( type, txt, style, fontFamily, info );
			writer.closeTag( "w:p" );
		}

	}

	private void outputText( int type, String txt, IStyle style,
			String fontFamily, HyperlinkInfo info )
	{
		writer.openTag( "w:p" );
		writer.openTag( "w:pPr" );

		if ( !style.getLineHeight( ).equalsIgnoreCase( "normal" ) )
		{
			writeSpacing( style.getLineHeight( ) );
		}

		writeAlign( style.getTextAlign( ), style.getDirection( ) );

		String val = parseColor( style.getBackgroundColor( ) );
		writeBackgroundColor( val );
		writer.openTag( "w:pBdr" );
		writeBorders( style );
		writer.closeTag( "w:pBdr" );
		writer.closeTag( "w:pPr" );
		writeTextInR( type, txt, style, fontFamily, info, false );
	}

	public void writeCaption( String txt )
	{
		writer.openTag( "w:p" );
		writer.openTag( "w:pPr" );
		writeAlign( "center", null );
		writer.closeTag( "w:pPr" );
		writer.openTag( "w:r" );
		writer.openTag( "w:rPr" );
		writeString( txt, null );
		writer.closeTag( "w:rPr" );
		writer.closeTag( "w:r" );
		writer.closeTag( "w:p" );
	}

	private void writeBackgroundColor( String color )
	{
		if ( color == null )
		{
			return;
		}

		writer.openTag( "w:shd" );
		writer.attribute( "w:val", "clear" );
		writer.attribute( "w:color", "auto" );
		writer.attribute( "w:fill", color );
		writer.closeTag( "w:shd" );
	}

	public void openHyperlink( HyperlinkInfo info )
	{
		if ( info == null )
		{
			return;
		}

		writer.openTag( "w:hlink" );

		if ( HyperlinkInfo.BOOKMARK == info.type )
		{

			writer.attribute( "w:bookmark", info.url );
		}
		else if ( HyperlinkInfo.HYPERLINK == info.type )
		{
			writer.attribute( "w:dest", info.url );
			if ( info.mark != null )
			{
				writer.attribute( "w:bookmark", info.mark );
			}
		}
		if ( info.toolTip != null )
		{
			writer.attribute( "w:screenTip", info.toolTip );
		}
	}

	public void closeHyperlink( HyperlinkInfo info )
	{
		if ( ( info == null ) || ( info.type == HyperlinkInfo.DRILL ) )
		{
			return;
		}

		writer.closeTag( "w:hlink" );
	}

	public void writeBookmark( String bm )
	{
		bm = WordUtil.validBookmarkName( bm );

		writer.openTag( "aml:annotation" );
		writer.attribute( "aml:id", BOOKMARK_ID );
		writer.attribute( "w:type", "Word.Bookmark.Start" );
		writer.attribute( "w:name", bm );
		writer.closeTag( "aml:annotation" );

		writer.openTag( "aml:annotation" );
		writer.attribute( "aml:id", BOOKMARK_ID );
		writer.attribute( "w:type", "Word.Bookmark.End" );
		writer.closeTag( "aml:annotation" );

		BOOKMARK_ID++;
	}

	public void writeHeading( String heading )
	{
		writer.openTag( "w:p" );
		writer.openTag( "w:pPr" );

		writer.openTag( "w:outlineLvl" );
		writer.attribute( "w:val", "1" );
		writer.closeTag( "w:outlineLvl" );

		writer.closeTag( "w:pPr" );
		writer.openTag( "w:bookmarkStart" );
		writer.attribute( "w:id", HEADING_ID );
		writer.attribute( "w:name", "TOC_TARGET_Heading" );
		writer.closeTag( "w:bookmarkStart" );
		writer.openTag( "w:bookmarkEnd" );
		writer.attribute( "w:id", HEADING_ID );
		writer.closeTag( "w:bookmarkEnd" );
		writer.openTag( "w:r" );
		writer.openTag( "w:rPr" );
		writer.closeTag( "w:rPr" );
		writer.openTag( "w:t" );
		writer.text( heading, true, false );
		writer.closeTag( "w:t" );
		writer.closeTag( "w:r" );
		writer.closeTag( "w:p" );

		HEADING_ID++;
	}

	public void close( )
	{
		writer.close( );

	}

	public void writeRunProperties( IStyle style, String fontFamily,
			boolean hyper )
	{
		String val = parseFont( fontFamily );
		// Font
		writer.openTag( "w:rFonts" );
		writer.attribute( "w:ascii", val );
		writer.attribute( "w:fareast", val );
		writer.attribute( "w:h-ansi", val );

		writer.attribute( "w:cs", val );
		writer.closeTag( "w:rFonts" );
		// Font size
		val = ridquote( style.getFontSize( ) );
		int size = Math.round( Float.parseFloat( val ) / 1000 ) * 2;
		String sizeValue = String.valueOf( size );
		writeAttrTag( "w:sz", sizeValue );
		writeAttrTag( "w:sz-cs", sizeValue );
		// letter spacing
		val = ridquote( style.getLetterSpacing( ) );
		writeAttrTag( "w:spacing", Math.round( Float.parseFloat( val ) / 1000 )
				* 20 + "" );

		// Font style
		val = ridquote( style.getTextUnderline( ) );
		if ( !"none".equalsIgnoreCase( val ) )
		{
			writeAttrTag( "w:u", "single" );
		}

		val = ridquote( style.getTextLineThrough( ) );
		if ( !"none".equalsIgnoreCase( val ) )
		{
			writeAttrTag( "w:strike", "on" );
		}

		val = ridquote( style.getFontStyle( ) );
		if ( !"normal".equalsIgnoreCase( val ) )
		{
			writeAttrTag( "w:i", "on" );
		}

		val = ridquote( style.getFontWeight( ) );
		if ( !"normal".equalsIgnoreCase( val ) )
		{
			writeAttrTag( "w:b", "on" );
		}

		// Text Color
		val = parseColor( style.getColor( ) );
		if ( val != null )
		{
			writeAttrTag( "w:color", val );
		}

		// deal with hyperlink
		if ( hyper )
		{
			writeAttrTag( "w:u", "single" );
			writeAttrTag( "w:color", "0000ff" );
		}
	}

	protected void writeAttrTag( String name, String val )
	{
		writer.openTag( name );
		writer.attribute( "w:val", val );
		writer.closeTag( name );
	}

	public void writePageBackground( String type, String data )
	{
		String color = null;

		if ( "color".equalsIgnoreCase( type ) )
		{
			color = parseColor( data );

			if ( color == null )
			{
				return;
			}
		}

		writer.openTag( "w:bgPict" );

		if ( "image".equalsIgnoreCase( type ) )
		{
			int imgId = getImageID( );
			writer.openTag( "w:binData" );
			writer.attribute( "w:name", "wordml://" + imgId + ".png" );
			writer.text( data );
			writer.closeTag( "w:binData" );
			writer.openTag( "w:background" );
			writer.attribute( "w:bgcolor", "white" );
			writer.attribute( "w:background", "wordml://" + imgId + ".png" );
			writer.closeTag( "w:background" );
		}
		else
		{
			writer.openTag( "w:background" );
			writer.attribute( "w:bgcolor", color );
			writer.closeTag( "w:background" );
		}

		writer.closeTag( "w:bgPict" );
	}

	protected void writeString( String txt, IStyle style )
	{
		if ( txt == null )
		{
			return;
		}
		if ( style != null )
		{
			if ( CSSConstants.CSS_CAPITALIZE_VALUE.equalsIgnoreCase( style
					.getTextTransform( ) ) )
			{
				txt = capitalize( txt );
			}
			else if ( CSSConstants.CSS_UPPERCASE_VALUE.equalsIgnoreCase( style
					.getTextTransform( ) ) )
			{
				txt = txt.toUpperCase( );
			}
			else if ( CSSConstants.CSS_LOWERCASE_VALUE.equalsIgnoreCase( style
					.getTextTransform( ) ) )
			{
				txt = txt.toLowerCase( );
			}
		}

		StringTokenizer st = new StringTokenizer( txt, "\n" );

		if ( !st.hasMoreTokens( ) && txt.length( ) != 0 )
		{
			// writer.openTag( "w:br" );
			// writer.closeTag( "w:br" );
		}

		writer.openTag( "w:t" );
		boolean nbreak = false;

		while ( st.hasMoreTokens( ) )
		{

			String row = "<![CDATA[" + st.nextToken( ) + "]]>";
			if ( nbreak )
			{
				row = "<w:br/>" + row;
			}
			else
			{
				nbreak = true;
			}

			writer.text( row, true, false );
		}

		writer.closeTag( "w:t" );
	}

	private String capitalize( String text )
	{
		boolean capitalizeNextChar = true;
		char[] array = text.toCharArray( );
		for ( int i = 0; i < array.length; i++ )
		{
			Character c = new Character( text.charAt( i ) );
			if ( splitChar.contains( c ) )
				capitalizeNextChar = true;
			else if ( capitalizeNextChar )
			{
				array[i] = Character.toUpperCase( array[i] );
				capitalizeNextChar = false;
			}
		}
		return new String( array );
	}

	private String ridquote( String val )
	{
		if ( val.charAt( 0 ) == '"' && val.charAt( val.length( ) - 1 ) == '"' )
		{
			return val.substring( 1, val.length( ) - 1 );
		}

		return val;
	}

	// generic font mapping
	private String parseFont( String fontFamily )
	{
		String val = ridquote( fontFamily );
		if ( "serif".equalsIgnoreCase( val ) )
			return "Times New Roman";
		if ( "sans-serif".equalsIgnoreCase( val ) )
			return "Arial";
		if ( "monospace".equalsIgnoreCase( val ) )
			return "Courier New";
		if ( "fantasy".equalsIgnoreCase( val ) )
			return "Times New Roman";
		if ( "cursive".equalsIgnoreCase( val ) )
			return "Times New Roman";
		return val;
	}

	// the valid color format is "rgb(0,0,0)"
	private String parseColor( String color )
	{
		if ( "transparent".equalsIgnoreCase( color ) || color == null )
		{
			return null;
		}

		String[] values = color.substring( color.indexOf( "(" ) + 1,
				color.length( ) - 1 ).split( "," );

		String value = "";

		for ( int i = 0; i < values.length; i++ )
		{
			try
			{
				String s = Integer.toHexString( ( Integer.parseInt( values[i]
						.trim( ) ) ) );

				if ( s.length( ) == 1 )
				{
					s = "0" + s;
				}

				value += s;
			}
			catch ( Exception e )
			{
				logger.log( Level.WARNING, e.getMessage( ), e );
				value = null;
			}
		}

		return value;
	}

	public void startTblRow( double height )
	{
		startTblRow( height, false, false );
	}

	/**
	 * 
	 * @param style
	 *            style of the row
	 * @param height
	 *            height of current row, if heigh equals 1 then ignore height
	 * @param type
	 *            header or normal
	 */

	public void startTblRow( double height, boolean isHeader,
			boolean repeatHeader )
	{
		writer.openTag( "w:tr" );

		// write the row height
		writer.openTag( "w:trPr" );

		if ( height != -1 )
		{
			writeAttrTag( "w:trHeight", height + "" );
		}

		// sign this row as header row
		// if value is "off",the header will be not repeated
		if ( isHeader )
		{
			if ( repeatHeader )
			{
				writeAttrTag( "w:tblHeader", "on" );
			}
			else
			{
				writeAttrTag( "w:tblHeader", "off" );
			}
		}

		writer.closeTag( "w:trPr" );
	}

	public void endTblRow( )
	{
		writer.closeTag( "w:tr" );
	}

	public void startTblCell( int w, IStyle style, SpanInfo info )
	{
		writer.openTag( "w:tc" );
		writer.openTag( "w:tcPr" );
		writer.openTag( "w:tcW" );
		writer.attribute( "w:w", w );
		writer.attribute( "w:type", "dxa" );
		writer.closeTag( "w:tcW" );

		if ( info != null )
		{
			if ( info.cs > 1 )
			{
				writeAttrTag( "w:gridSpan", info.cs + "" );
			}

			if ( info.start )
			{
				writeAttrTag( "w:vmerge", "restart" );
			}
			else
			{
				writer.openTag( "w:vmerge" );
				writer.closeTag( "w:vmerge" );
			}
		}

		writeCellProp( style );
		writer.closeTag( "w:tcPr" );

		String align = style.getTextAlign( );
		if ( null == align )
		{
			return;
		}
		String direction = style.getDirection( ); // bidi_hcg
		if ( CSSConstants.CSS_LEFT_VALUE.equals( align ) )
		{
			if ( !CSSConstants.CSS_RTL_VALUE.equals( direction ) )
				return;
		}

		writer.openTag( "w:pPr" );
		writeAlign( align, direction );
		writer.closeTag( "w:pPr" );

	}

	public void writeSpanCell( SpanInfo info )
	{
		writer.openTag( "w:tc" );
		writer.openTag( "w:tcPr" );
		writer.openTag( "w:tcW" );
		writer.attribute( "w:w", info.width );
		writer.attribute( "w:type", "dxa" );
		writer.closeTag( "w:tcW" );

		if ( info.cs > 1 )
		{
			writeAttrTag( "w:gridSpan", info.cs + "" );
		}

		if ( info.start )
		{
			writeAttrTag( "w:vmerge", "restart" );
		}
		else
		{
			writer.openTag( "w:vmerge" );
			writer.closeTag( "w:vmerge" );
		}

		writeCellProp( info.style );

		writer.closeTag( "w:tcPr" );
		insertHiddenP( );
		writer.closeTag( "w:tc" );
	}

	/**
	 * If the cell properties is not set, then check the row properties and
	 * write those properties.
	 * 
	 * @param style
	 *            this cell style ��
	 */

	private void writeCellProp( IStyle style )
	{
		// A cell background color may inherit from row background,
		// so we should get the row background color here,
		// if the cell background is transparent
		String bgColor = parseColor( style.getBackgroundColor( ) );

		if ( bgColor != null )
		{
			writer.openTag( "w:shd" );
			writer.attribute( "w:val", "clear" );
			writer.attribute( "w:color", "auto" );
			writer.attribute( "w:fill", bgColor );
			writer.closeTag( "w:shd" );
		}

		writer.openTag( "w:tcBorders" );
		writeBorders( style );
		writer.closeTag( "w:tcBorders" );

		String verticalAlign = style.getVerticalAlign( );
		if ( verticalAlign != null )
		{
			writer.openTag( "w:vAlign" );
			writer.attribute( "w:val",
					convertToWordVerticalAlign( verticalAlign ) );
			writer.closeTag( "w:vAlign" );
		}

	}

	private String convertToWordVerticalAlign( String align )
	{
		if ( "middle".equals( align ) )
		{
			return "center";
		}
		return align;
	}

	public void endTblCell( boolean empty )
	{
		if ( empty )
		{
			insertHiddenP( );
		}

		writer.closeTag( "w:tc" );
	}

	public void startPg( )
	{
		writer.openTag( "wx:sect" );
	}

	public void endPg( )
	{
		writer.closeTag( "wx:sect" );
	}

	public void end( )
	{
		writer.closeTag( "w:body" );
		writer.closeTag( "w:wordDocument" );
		writer.close( );

	}

	public void startHeader( boolean showHeaderOnFirst )
	{
		writer.openTag( "w:hdr" );
		if ( showHeaderOnFirst )
		{
			writer.attribute( "w:type", "first" );
			writer.openTag( "w:p" );
			writer.openTag( "w:r" );
			writer.closeTag( "w:r" );
			writer.closeTag( "w:p" );
		}

		writer.attribute( "w:type", "odd" );
	}

	public void endHeader( )
	{
		writer.closeTag( "w:hdr" );
	}

	public void startFooter( )
	{
		writer.openTag( "w:ftr" );
		writer.attribute( "w:type", "odd" );
	}

	public void endFooter( )
	{
		writer.closeTag( "w:ftr" );

	}

	public void writePgProp( int pageHeight, int pageWidth, int headerHeight,
			int footerHeight, int topMargin, int bottomMargin, int leftMargin,
			int rightMargin, String orient )
	{

		writer.openTag( "w:pgSz" );
		writer.attribute( "w:w", pageWidth );
		writer.attribute( "w:h", pageHeight );
		writer.attribute( "w:orient", orient );
		writer.closeTag( "w:pgSz" );

		writer.openTag( "w:pgMar" );
		writer.attribute( "w:top", topMargin );
		writer.attribute( "w:bottom", bottomMargin );
		writer.attribute( "w:left", leftMargin );
		writer.attribute( "w:right", rightMargin );
		writer.attribute( "w:header", topMargin );
		writer.attribute( "w:footer", bottomMargin );
		writer.closeTag( "w:pgMar" );
	}

	public void writeAutoText( int type )
	{
		writer.openTag( "w:instrText" );

		if ( type == IAutoTextContent.PAGE_NUMBER )
		{
			writer.text( "PAGE" );
		}
		else if ( type == IAutoTextContent.TOTAL_PAGE )
		{
			writer.text( "NUMPAGES" );
		}

		writer.closeTag( "w:instrText" );
	}

	public void endParagraph( )
	{
		writer.closeTag( "w:p" );
	}

	public void writeEmptyCell( )
	{
		writer.openTag( "w:tc" );
		writer.openTag( "w:tcPr" );
		writer.openTag( "w:tcW" );
		writer.attribute( "w:w", 0 );
		writer.attribute( "w:type", "dxa" );
		writer.closeTag( "w:tcW" );
		writer.closeTag( "w:tcPr" );
		insertHiddenP( );
		writer.closeTag( "w:tc" );
	}

	public void writeTC( String tocText, int level )
	{
		writer.openTag( "w:p" );

		writer.openTag( "w:rPr" );
		writer.openTag( "w:vanish" );
		writer.closeTag( "w:vanish" );
		writer.closeTag( "w:rPr" );

		writer.openTag( "aml:annotation" );
		writer.attribute( "aml:id", BOOKMARK_ID );
		writer.attribute( "w:type", "Word.Bookmark.Start" );
		writer.attribute( "w:name", "_Toc" + tocText );
		writer.closeTag( "aml:annotation" );

		writer.openTag( "aml:annotation" );
		writer.attribute( "aml:id", BOOKMARK_ID++ );
		writer.attribute( "w:type", "Word.Bookmark.End" );
		writer.closeTag( "aml:annotation" );
		writer.openTag( "w:r" );
		writer.openTag( "w:fldChar" );
		writer.attribute( "w:fldCharType", "begin" );
		writer.closeTag( "w:fldChar" );
		writer.closeTag( "w:r" );

		writer.openTag( "w:r" );
		writer.openTag( "w:instrText" );
		writer.text( " TC \"" + tocText + "\"" + " \\f C \\l \""
				+ String.valueOf( level ) + "\"" );
		writer.closeTag( "w:instrText" );
		writer.closeTag( "w:r" );

		writer.openTag( "w:r" );
		writer.openTag( "w:fldChar" );
		writer.attribute( "w:fldCharType", "end" );
		writer.closeTag( "w:fldChar" );
		writer.closeTag( "w:r" );

		writer.closeTag( "w:p" );
	}

	/**
	 * @param text
	 * @param rtl
	 * @return
	 * 
	 * @author bidi_hcg
	 */
	private void buildBidiChunks( String text, boolean rtl )
	{
		Bidi bidiObj = new Bidi( text, rtl
				? Bidi.DIRECTION_RIGHT_TO_LEFT
				: Bidi.DIRECTION_LEFT_TO_RIGHT );
		int nRuns = bidiObj.getRunCount( );
		if ( nRuns != bidiChunks.length )
		{
			bidiChunks = new String[nRuns];
			bidiLevels = new int[nRuns];
		}
		for ( int i = 0; i < nRuns; i++ )
		{
			bidiChunks[i] = text.substring( bidiObj.getRunStart( i ), bidiObj
					.getRunLimit( i ) );
			bidiLevels[i] = bidiObj.getRunLevel( i ) & 1;
		}
	}
}
