/*******************************************************************************
 * Copyright (c) 2004 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.report.model.api.metadata;

import org.eclipse.birt.report.model.api.util.StringUtil;

/**
 * Represents the definition of argument. The argument definition includes the
 * data type, internal name, and display name.
 */

public class ArgumentInfo implements IArgumentInfo
{

	private final IArgumentInfo arguInfo;

	public ArgumentInfo( )
	{
		arguInfo = new org.eclipse.birt.report.model.metadata.ArgumentInfo( );
	}

	public String getName( )
	{
		return arguInfo.getName( );
	}

	/**
	 * Returns the display name for the property if the resource key of display
	 * name is defined. Otherwise, return empty string.
	 * 
	 * @return the user-visible, localized display name for the property
	 */

	public String getDisplayName( )
	{
		String retValue = arguInfo.getDisplayName( );
		return retValue != null ? retValue : ""; //$NON-NLS-1$
	}

	/**
	 * Sets the internal name of the property.
	 * 
	 * @param theName
	 *            the internal property name
	 */

	protected void setName( String theName )
	{
		( (org.eclipse.birt.report.model.metadata.ArgumentInfo) arguInfo )
				.setName( theName );
	}

	/**
	 * Returns the resource key for the display name.
	 * 
	 * @return The display name message ID.
	 */

	public String getDisplayNameKey( )
	{
		String retValue = arguInfo.getDisplayNameKey( );
		return retValue != null ? retValue : ""; //$NON-NLS-1$
	}

	/**
	 * Sets the message ID for the display name.
	 * 
	 * @param id
	 *            message ID for the display name
	 */

	protected void setDisplayNameKey( String id )
	{
		( (org.eclipse.birt.report.model.metadata.ArgumentInfo) arguInfo )
				.setDisplayNameKey( id );
	}

	/**
	 * Returns the script type of this argument.
	 * 
	 * @return the script type to set
	 */

	public String getType( )
	{
		return arguInfo.getType( );
	}

	/**
	 * Returns the class type of this argument.
	 * 
	 * @return the class type to set
	 */

	public IClassInfo getClassType( )
	{
		return null;
	}

	/**
	 * Sets the script type of this argument.
	 * 
	 * @param type
	 *            the script type to set
	 */

	protected void setType( String type )
	{
		( (org.eclipse.birt.report.model.metadata.ArgumentInfo) arguInfo )
				.setType( type );
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */

	public String toString( )
	{
		if ( !StringUtil.isBlank( getName( ) ) )
			return getName( );
		return super.toString( );
	}
}