/*******************************************************************************
 * Copyright (c) 2004 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.report.model.api.util;

import org.eclipse.birt.report.model.core.namespace.NameExecutor;

/**
 * Utility class to provide some methods about cube.
 */
public class CubeUtil
{

	private final static int DIMENSION_INDEX = 0;
	private final static int LEVEL_INDEX = 1;

	/**
	 * Splits a full name of the level element to a <code>String</code> array
	 * of length 2. The first member of the array is the name of the dimension
	 * element and the second is the name of the level element.
	 * 
	 * @param levelName
	 * @return
	 */
	public static String[] splitLevelName( String levelName )
	{
		String[] results = new String[2];
		results[DIMENSION_INDEX] = null;
		results[LEVEL_INDEX] = null;

		// if level name is null, we should do nothing
		if ( levelName != null )
		{
			int index = levelName.lastIndexOf( NameExecutor.NAME_SEPARATOR );
			if ( index > -1 )
			{
				String dimension = levelName.substring( 0, index );
				String level = levelName.substring( index + 1 );
				results[DIMENSION_INDEX] = dimension;
				results[LEVEL_INDEX] = level;
			}
			else
			{
				results[DIMENSION_INDEX] = null;
				results[LEVEL_INDEX] = levelName;
			}
		}

		return results;
	}
}
